Type: Package
Package: admiralophtha
Title: ADaM in R Asset Library - Ophthalmology
Version: 1.5.0
Authors@R: c(
    person("Edoardo", "Mancini", , "edoardo.mancini@roche.com", role = c("aut", "cre"),
           comment = c(ORCID = "0009-0006-4899-8641")),
    person("Yuki", "Matsunaga", , "yuki.matsunaga@novartis.com", role = "aut"),
    person("F. Hoffmann-La Roche AG", role = c("cph", "fnd")),
    person("Novartis AG", role = c("cph", "fnd"))
  )
Description: Aids the programming of Clinical Data Standards Interchange
    Consortium (CDISC) compliant Ophthalmology Analysis Data Model (ADaM)
    datasets in R. ADaM datasets are a mandatory part of any New Drug or
    Biologics License Application submitted to the United States Food and
    Drug Administration (FDA). Analysis derivations are implemented in
    accordance with the "Analysis Data Model Implementation Guide" (CDISC
    Analysis Data Model Team, 2021,
    <https://www.cdisc.org/standards/foundational/adam/adamig-v1-3-release-package>).
License: Apache License (>= 2)
URL: https://pharmaverse.github.io/admiralophtha/,
        https://github.com/pharmaverse/admiralophtha/
BugReports: https://github.com/pharmaverse/admiralophtha/issues/
Depends: R (>= 4.1)
Imports: admiral (>= 1.5.0), admiraldev (>= 1.5.0), dplyr (>= 1.1.1),
        hms (>= 0.5.3), lifecycle (>= 0.1.0), lubridate (>= 1.7.4),
        magrittr (>= 1.5), purrr (>= 0.3.3), rlang (>= 0.4.4), stringr
        (>= 1.4.0), tidyr (>= 1.0.2), tidyselect (>= 1.1.0)
Suggests: devtools, diffdf, DT, here, htmltools, knitr, lintr, methods,
        miniUI, pharmaversesdtm (>= 1.3.0), pkgdown, rmarkdown,
        roxygen2, spelling, testthat (>= 3.0.0), tibble, usethis, withr
VignetteBuilder: knitr
Config/roxygen2/markdown: TRUE
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
LazyData: true
NeedsCompilation: no
Packaged: 2026-06-12 14:47:56 UTC; mancinie
Author: Edoardo Mancini [aut, cre] (ORCID:
    <https://orcid.org/0009-0006-4899-8641>),
  Yuki Matsunaga [aut],
  F. Hoffmann-La Roche AG [cph, fnd],
  Novartis AG [cph, fnd]
Maintainer: Edoardo Mancini <edoardo.mancini@roche.com>
Repository: CRAN
Date/Publication: 2026-06-12 15:20:15 UTC
Built: R 4.6.0; ; 2026-06-12 15:51:05 UTC; unix
