---
title: "Get Started"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Get Started}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

As this is a package extension, if you are new to `{admiral}` then the best place to first start
reading would be this [Get Started](https://pharmaverse.github.io/admiral/cran-release/articles/admiral.html)
guide. This extension package follows the same main idea and conventions, and re-uses many
functions from `{admiral}`, so it is important to thoroughly understand these to be able to
use `{admiralophtha}`.

# What does {admiralophtha} offer?

The contents of `{admiralophtha}` can be broadly divided into three categories:

- **Derivations** (i.e. functions): these follow the same conventions as `{admiral}` but are focused on ophthalmology-specific needs. Please see the Reference page for more details.
- **ADaM Templates**: these showcase ways to apply `{admiral}`, `{admiralophtha}` and Tidyverse functions to create ophthalmology-specific ADaMs. You can see the available templates in `{admiralophtha}` by running: `list_all_templates(package = "admiralophtha")` in the console.
- **User Guides** (i.e. vignettes): these are documentation pages which walk users through each step of a template, or discuss other ophthalmology-specific topics such as ophthalmology standards. Please see the User Guides section for more details.

# Starting a Script

For the ophthalmology ADaM datasets, an overview of the flow and example function calls
for the most common steps are provided by the following vignettes:

- [Creating ADBCVA](adbcva.html)
- [Creating ADOE](adoe.html)
- [Creating ADVFQ](advfq.html)

# Support

Support is provided via the [admiral Slack
channel](https://pharmaverse.slack.com/).
